@@ -22,6 +22,92 @@
 /* g_free already checks if x is NULL */
 #define free_if_empty(x) g_free (x)
 
+/* find language in a list of GnomeDesktopEntryI18N's*/
+static GnomeDesktopEntryI18N *
+find_lang(GList *list, char *lang)
+{
+	for(;list;list=list->next) {
+		GnomeDesktopEntryI18N *e = list->data;
+		if((!lang && !e->lang) ||
+		   (lang && e->lang && strcmp(e->lang,lang)==0)) {
+			return e;
+		}
+	}
+	return NULL;
+}
+
+/* add a comment or name to a language list */
+static GList *
+add_comment_or_name(GList *list, char *lang, char *name, char *comment)
+{
+	GnomeDesktopEntryI18N *entry;
+	
+	entry = find_lang(list,lang);
+	if(!entry) {
+		entry = g_new0(GnomeDesktopEntryI18N,1);
+		entry->lang = g_strdup(lang);
+		list = g_list_prepend(list,entry);
+	}
+	
+	if(name) {
+		if(entry->name)
+			g_free(entry->name);
+		entry->name = g_strdup(name);
+	}
+	if(comment) {
+		if(entry->comment)
+			g_free(entry->comment);
+		entry->comment = g_strdup(comment);
+	}
+	return list;
+}
+
+/*get the lang out of key, (modifies key!) */
+#define GET_LANG(lang,key,len) 				\
+	if(key[len]=='[') {				\
+		char *p = strchr(&key[(len)+1],']');	\
+		if(p) {					\
+			*p = '\0';			\
+			lang = &key[(len)+1];		\
+		}					\
+	}
+
+/*read the names and comments from the desktop file*/
+static GList *
+read_names_and_comments(const char *file, int is_kde)
+{
+	GList *i18n_list = NULL;
+	
+	gpointer iterator;
+	char *key,*value;
+	char *prefix;
+	
+	gnome_config_push_prefix ("");
+	if(!is_kde) {
+		prefix = g_strconcat ("=", file, "=/Desktop Entry", NULL);
+	} else {
+		prefix = g_strconcat ("=", file, "=/KDE Desktop Entry", NULL);
+	}
+	iterator = gnome_config_init_iterator(prefix);
+	g_free(prefix);
+	gnome_config_pop_prefix ();
+	/*it HAS to be there*/
+	g_assert(iterator);
+	while ((iterator = gnome_config_iterator_next(iterator, &key, &value))) {
+		if(strncmp(key,"Name",4)==0) {
+			char *lang = NULL;
+			GET_LANG(lang,key,4)
+			i18n_list = add_comment_or_name(i18n_list,lang,value,NULL);
+		} else if(strncmp(key,"Comment",7)==0) {
+			char *lang = NULL;
+			GET_LANG(lang,key,7)
+			i18n_list = add_comment_or_name(i18n_list,lang,NULL,value);
+		}
+		g_free(key);
+		g_free(value);
+	}
+	return i18n_list;
+}
 
 /**
  * gnome_desktop_entry_load_flags_conditional:
@@ -54,8 +140,10 @@ gnome_desktop_entry_load_flags_conditional (const char *file,
 	char *p = NULL;
 	gboolean is_kde = FALSE;
 	
+	GList *i18n_list = NULL;
+	
 	g_assert (file != NULL);
-
+	
 	prefix = g_strconcat ("=", file, "=/Desktop Entry/", NULL);
 	gnome_config_push_prefix (prefix);
 	g_free (prefix);
@@ -102,7 +190,7 @@ gnome_desktop_entry_load_flags_conditional (const char *file,
 			g_free (p);
 	}
 	
-	newitem = g_new (GnomeDesktopEntry, 1);
+	newitem = g_new0 (GnomeDesktopEntry, 1);
 
 	newitem->name          = name;
 	newitem->comment       = gnome_config_get_translated_string ("Comment");
@@ -144,9 +232,14 @@ gnome_desktop_entry_load_flags_conditional (const char *file,
 		if(icon_base) g_free(icon_base);
 		newitem->icon = NULL;
 	}
-	gnome_config_pop_prefix ();
 	
-	if (clean_from_memory_after_load){
+	gnome_config_pop_prefix ();
+
+	/*get us the Names and comments of different languages*/
+	i18n_list = read_names_and_comments (file, is_kde);
+	gnome_desktop_entry_set_i18n_list (newitem, i18n_list);
+	
+	if (clean_from_memory_after_load) {
 		prefix = g_strconcat ("=", file, "=", NULL);
 		gnome_config_drop_file (prefix);
 		g_free (prefix);
@@ -209,6 +302,7 @@ void
 gnome_desktop_entry_save (GnomeDesktopEntry *dentry)
 {
 	char *prefix;
+	GList *i18n_list = NULL,*li;
 	
 /* XXX:this should have same clean_from_memory logic as above maybe??? */
 	
@@ -222,9 +316,32 @@ gnome_desktop_entry_save (GnomeDesktopEntry *dentry)
 	gnome_config_push_prefix (prefix);
 	g_free (prefix);
 
+	/*set the names and comments from our i18n list*/
+	i18n_list = gnome_desktop_entry_get_i18n_list(dentry);
+	for (li=i18n_list; li; li=li->next) {
+		GnomeDesktopEntryI18N *e = li->data;
+		if (e->name) {
+			char *key;
+			if (e->lang)
+				key = g_strdup_printf("Name[%s]",e->lang);
+			else
+				key = g_strdup("Name");
+			gnome_config_set_string (key, e->name);
+			g_free(key);
+		}
+		if (e->comment) {
+			char *key;
+			if (e->lang)
+				key = g_strdup_printf("Comment[%s]",e->lang);
+			else
+				key = g_strdup("Comment");
+			gnome_config_set_string (key, e->comment);
+		}
+	}
+
+	/*set these two as well, just to override potential conflicts*/
 	if (dentry->name)
 		gnome_config_set_translated_string ("Name", dentry->name);
-
 	if (dentry->comment)
 		gnome_config_set_translated_string ("Comment", dentry->comment);
 
@@ -268,7 +385,7 @@ gnome_desktop_entry_save (GnomeDesktopEntry *dentry)
 void
 gnome_desktop_entry_free (GnomeDesktopEntry *item)
 {
-	if(item){
+	if (item) {
 		free_if_empty (item->name);
 		free_if_empty (item->comment);
 		g_strfreev (item->exec);
@@ -278,6 +395,8 @@ gnome_desktop_entry_free (GnomeDesktopEntry *item)
 		free_if_empty (item->type);
 		free_if_empty (item->location);
 		free_if_empty (item->geometry);
+		gnome_desktop_entry_free_i18n_list (gnome_desktop_entry_get_i18n_list(item));
+		g_dataset_destroy (item);
 		g_free (item);
 	}
 }
@@ -355,6 +474,33 @@ gnome_desktop_entry_sub_kde_arg (GnomeDesktopEntry *item, gchar *arg)
 		return NULL;
 }
 
+static char *
+join_with_quotes(char *argv[])
+{
+	int i;
+	char *ret;
+	GString *gs = g_string_new("");
+	for(i=0;argv[i];i++) {
+		char *p = strchr(argv[i],'\'');
+		if(!p) {
+			g_string_sprintfa(gs,"%s'%s'",i==0?"":" ",argv[i]);
+		} else {
+			char *s = g_strdup(argv[i]);
+			g_string_sprintfa(gs,"%s'",i==0?"":" ");
+			while((p = strchr(s,'\''))) {
+				*p='\0';
+				g_string_sprintfa(gs,"%s'\\''",s);
+				s = p+1;
+			}
+			g_string_sprintfa(gs,"%s'",s);
+			g_free(s);
+		}
+	}
+	ret = gs->str;
+	g_string_free(gs,FALSE);
+	return ret;
+}
+
 /**
  * gnome_desktop_entry_launch_with_args:
  * @item: a gnome desktop entry.
@@ -422,7 +568,7 @@ gnome_desktop_entry_launch_with_args (GnomeDesktopEntry *item, int the_argc, cha
 		
 		argv[argc] = NULL;
 		
-		exec_str = g_strjoinv (" ", (char **)argv);
+		exec_str = join_with_quotes ((char **)argv);
 		
 		/* clean up */
 		if (term_argc && term_argv != xterm_argv)
@@ -485,14 +631,20 @@ gnome_desktop_entry_destroy (GnomeDesktopEntry *item)
  * gnome_desktop_entry_copy:
  * @source: a GnomeDesktop entry.
  *
- * Returns a copy of the @source GnomeDesktopEntry
+ * Description: Makes a copy of a @source GnomeDesktopEntry
+ *
+ * Returns: A copy of the @source GnomeDesktopEntry
  */
 GnomeDesktopEntry *
 gnome_desktop_entry_copy (GnomeDesktopEntry * source)
 {
 	GnomeDesktopEntry * newitem;
+	GList *i18n_list = NULL;
+	GList *new_i18n_list = NULL;
+	GList *li;
 	
 	g_return_val_if_fail (source != NULL, NULL);
+	
 	newitem = g_new (GnomeDesktopEntry, 1);
 	
 	newitem->name          = g_strdup (source->name);
@@ -508,6 +660,77 @@ gnome_desktop_entry_copy (GnomeDesktopEntry * source)
 	newitem->location      = g_strdup (source->location);
 	newitem->icon          = g_strdup (source->icon);
 	newitem->is_kde	       = source->is_kde;
+
+	i18n_list = gnome_desktop_entry_get_i18n_list(source);
+	for (li=i18n_list; li; li=li->next) {
+		GnomeDesktopEntryI18N *e = li->data;
+		GnomeDesktopEntryI18N *ne = g_new0(GnomeDesktopEntryI18N,1);
+		ne->lang = g_strdup (e->lang);
+		ne->name = g_strdup (e->name);
+		ne->comment = g_strdup (e->comment);
+		new_i18n_list = g_list_prepend(new_i18n_list,ne);
+	}
+	gnome_desktop_entry_set_i18n_list(source, new_i18n_list);
 	
 	return newitem;
 }
+
+/**
+ * gnome_desktop_entry_get_i18n_list:
+ * @item: a GnomeDesktopEntry.
+ *
+ * Description: Gets a list of the GnomeDesktopEntryI18N structures
+ * with the translations for @item. It's the actual one that's stored
+ * with the entry, not a copy. If you modify it you need to do a
+ * #gnome_desktop_entry_set_i18n_list.
+ *
+ * Returns: A GList * of GnomeDesktopEntryI18N's
+ */
+GList *
+gnome_desktop_entry_get_i18n_list(GnomeDesktopEntry *item)
+{
+	return g_dataset_get_data(item,"i18n_list");
+}
+
+/**
+ * gnome_desktop_entry_get_i18n_list:
+ * @item: a GnomeDesktopEntry.
+ * @list: a list of GnomeDesktopEntryI18N's
+ *
+ * Description: Sets the list of translations associated with this
+ * particular entry to @list. It does not free the current one, so
+ * if you want to free that one use #gnome_desktop_entry_get_i18n_list
+ * and #gnome_desktop_entry_free_i18n_list, first. This is so that
+ * you can get the list with #gnome_desktop_entry_get_i18n_list, modify
+ * it and set it back.
+ *
+ * Returns:
+ */
+void
+gnome_desktop_entry_set_i18n_list(GnomeDesktopEntry *item, GList *list)
+{
+	g_dataset_set_data(item,"i18n_list",list);
+}
+
+/**
+ * gnome_desktop_entry_free_i18n_list:
+ * @list: a list of GnomeDesktopEntryI18N's
+ *
+ * Description: A utility function for freeing a GList of
+ * GnomeDesktopEntryI18N's.
+ *
+ * Returns:
+ */
+void
+gnome_desktop_entry_free_i18n_list(GList *list)
+{
+	GList *li;
+	for (li=list; li; li=li->next) {
+		GnomeDesktopEntryI18N *e = li->data;
+		free_if_empty (e->lang);
+		free_if_empty (e->name);
+		free_if_empty (e->comment);
+		g_free(e);
+	}
+	if(list) g_list_free(list);
+}
