@@ -5,14 +5,13 @@
 #include <glib.h>
 BEGIN_GNOME_DECLS
 
-enum _GnomeTriggerType {
+typedef enum _GnomeTriggerType {
 	GTRIG_NONE,
 	GTRIG_FUNCTION,
 	GTRIG_COMMAND,
 	GTRIG_MEDIAPLAY
-};
+} GnomeTriggerType;
 
-typedef enum _GnomeTriggerType GnomeTriggerType;
 typedef void (*GnomeTriggerActionFunction)(char *msg, char *level, char *supinfo[]);
 
 struct _GnomeTrigger {
