@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; c-basic-offset: 2; -*- */
 #include <config.h>
 
 /* needed for sigaction and friends under 'gcc -ansi -pedantic' on 
@@ -17,7 +18,7 @@ int retval = 1;
 
 int main(int argc, char *argv[])
 {
-  GtkWidget *mainwin;
+  GtkWidget *mainwin, *urlbtn;
   gchar* msg;
   struct sigaction sa;
   poptContext ctx;
@@ -40,12 +41,12 @@ int main(int argc, char *argv[])
   args = poptGetArgs(ctx);
   if (args && args[0] && args[1])
     {
-      msg = g_strdup_printf(_("Application \"%s\" (process %d) has crashed due to a bug in the software.\n(%s)\nFor more information, see http://www.gnome.org/application_crashed.shtml"),
-                       args[0], getppid(), g_strsignal(atoi(args[1])));
+      msg = g_strdup_printf(_("Application \"%s\" (process %d) has crashed\ndue to a fatal error.\n(%s)"),
+                            args[0], getppid(), g_strsignal(atoi(args[1])));
     }
   else
     {
-      fprintf(stderr, "Usage: gnome_segv appname signum\n");
+      fprintf(stderr, _("Usage: gnome_segv appname signum\n"));
       return 1;
     }
 
@@ -56,6 +57,11 @@ int main(int argc, char *argv[])
                                   GNOME_STOCK_BUTTON_CLOSE,
                                   NULL);
 
+  urlbtn = gnome_href_new("http://www.gnome.org/application_crashed.shtml",
+                          _("Please visit the GNOME Application Crash page for more information"));
+  gtk_widget_show(urlbtn);
+  gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(mainwin)->vbox), urlbtn);
+
   g_free(msg);
 
   gnome_dialog_run(GNOME_DIALOG(mainwin));
