@@ -406,14 +406,15 @@ void about_cb(GtkWidget *widget, gpointer data)
 {
   GtkWidget *about;
   const gchar *authors[] = {"Max Watson", NULL};
-  
-  about = gnome_about_new ( _("WIN_HINTS Test"), "0.1",
-			   _("Copyright (C)1998"),
-			   authors,
-			   _("Simple test app to check how the WIN_HINTS work. And to test the gnome_win_hints_* functions for errors. :)"),
-			   NULL);
+
+  about = gnome_about_new (_("WIN_HINTS Test"), "0.1",
+                           _("Copyright (C) 1998"),
+                           authors,
+                           _("Simple test app to check how the WIN_HINTS work.  "
+                             "And to test the gnome_win_hints_* functions for errors. :)"),
+                           NULL);
   gtk_widget_show (about);
-  
+
   return;
-}                                
+}
 
