@@ -58,11 +58,9 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, struct vt_line *b
 {
   int i;
   int run, commonrun;
-  int runstart, commonrunstart;
+  int runstart;
   char *p;
-  uint32 attr, newattr, oldattr, oldchar, newchar, oldback, newback, lastchar;
-  uint32 c;
-  int ch;
+  uint32 attr, newattr, oldattr, oldchar, newchar, oldback, lastchar;
   /*  struct vt_line *bl;*/
   int sx, ex;			/* start/end selection */
   int force;
@@ -150,17 +148,17 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, struct vt_line *b
     else
       newchar = lastchar;
 
-    lastchar = newchar & VTATTR_MASK;
-
     /* check for selected block */
     if (i >= sx && i < ex) {
       newchar ^= VTATTR_REVERSE;
     }
 
+    /* used in both if cases */
+    newattr = newchar & VTATTR_MASK;
+
     /* if there are no changes, quit right here ... */
     if (oldchar != newchar || force) {
       bl->data[i] = newchar;
-      newattr = newchar & VTATTR_MASK;
       oldattr = oldchar & VTATTR_MASK;
       if (run) {
 	if (newattr == attr) {
@@ -203,7 +201,7 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, struct vt_line *b
 #ifdef VT_THRESHHOLD
 	/* check for runs of common characters, if they are short, then
 	   use them */
-	if (commonrun>VT_THRESHHOLD) {
+	if (commonrun>VT_THRESHHOLD || (newattr!=attr)) {
 	  vx->draw_text(vx->vt.user_data, bl,
 			line, runstart, run, attr);
 	  run=0;
@@ -218,6 +216,7 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, struct vt_line *b
 #endif
       }
     }
+    lastchar = newchar & VTATTR_MASK;
   }
 
   if (run) {
@@ -1107,7 +1106,7 @@ vt_select_block(struct _vtx *vx, int size, int sx, int sy, int ex, int ey, int *
   }
 
   /* makes a rough assumption about the buffer size needed */
-  if ( (data = g_malloc(size*(ey-sy+1)*(vx->vt.width+20))) == 0 ) {
+  if ( (data = g_malloc(size*(ey-sy+1)*(vx->vt.width+20)+1)) == 0 ) {
     *len = 0;
     printf("ERROR: Cannot g_malloc selection buffer\n");
     return 0;
@@ -1156,6 +1155,7 @@ vt_select_block(struct _vtx *vx, int size, int sx, int sy, int ex, int ey, int *
   }
 
   *len = (out-data)/size;
+  *out = 0;
 
   d(printf("selected text = \n");
     fwrite(data, out-data, 1, stdout);
